try:
    from django.urls import include, path
except ImportError:
    from django.conf.urls import include, url as path

from rest_framework.routers import DefaultRouter

from flexi_tag.resources.views import TaggedItemViewSet, TagViewSet

router = DefaultRouter()
router.register(
    r"tags",
    TagViewSet,
    basename="tags",
)
router.register(
    r"tagged-items",
    TaggedItemViewSet,
    basename="tagged-items",
)

app_name = "flexi_tag"


urlpatterns = [path("", include(router.urls))]



from django.test import TestCase
from model_mommy import mommy

from flexi_tag.exceptions import TagValidationException
from flexi_tag.models import Tag, TaggedItem
from flexi_tag.service import TagService


class TagServiceTestCase(TestCase):
    def setUp(self):
        self.service = TagService()

    def test_create_instance(self):
        tag_name = "test_tag"
        tag = self.service.create_instance(tag_name)

        self.assertEqual(tag.name, tag_name)

    def test_update_instance(self):
        tag = mommy.make(Tag, name="old_name")
        tagged_item = mommy.make(TaggedItem, tag=tag, name="old_name")

        new_name = "updated_name"
        self.service.update_instance(tag, new_name)

        tag.refresh_from_db()
        self.assertEqual(tag.name, new_name)

        tagged_item.refresh_from_db()
        self.assertEqual(tagged_item.name, new_name)

    def test_delete_instance(self):
        tag = mommy.make(Tag, name="test_tag")

        self.service.delete_instance(tag)
        self.assertEqual(Tag.objects.filter(id=tag.id).count(), 0)

    def test_create_instance_with_duplicate_name(self):
        tag_name = "test_tag"
        mommy.make(Tag, name=tag_name)

        with self.assertRaises(TagValidationException):
            self.service.create_instance(tag_name)

    def test_update_instance_with_duplicate_name(self):
        mommy.make(Tag, name="existing_tag")
        tag_to_update = mommy.make(Tag, name="tag_to_update")

        with self.assertRaises(TagValidationException):
            self.service.update_instance(tag_to_update, "existing_tag")



from django.contrib.contenttypes.models import ContentType
from django.db.models import QuerySet
from django.test import TestCase
from mock import MagicMock, patch

from flexi_tag.models import Tag
from flexi_tag.utils.service import TaggableService


class TaggableServiceTestCase(TestCase):
    def setUp(self):
        self.tag1 = Tag.objects.create(name="test-tag-1")
        self.tag2 = Tag.objects.create(name="test-tag-2")

        self.instance = MagicMock()
        self.instance.pk = 1
        self.instance._meta = MagicMock()
        self.instance._meta.app_label = "test_app"
        self.instance._meta.model_name = "test_model"

        self.content_type = ContentType.objects.get_or_create(
            app_label=self.instance._meta.app_label,
            model=self.instance._meta.model_name,
        )[0]

        self.service = TaggableService()

        self.queryset = MagicMock(spec=QuerySet)

    @patch("flexi_tag.models.TaggedItem.objects.create")
    def test_add_tag(self, mock_create):
        self.service.add_tag(self.instance, self.tag1)

        mock_create.assert_called_once_with(
            tag=self.tag1, content_object=self.instance, name=self.tag1.name
        )

    @patch("flexi_tag.models.TaggedItem.objects.filter")
    def test_remove_tag(self, mock_filter):
        mock_queryset = MagicMock()
        mock_filter.return_value = mock_queryset

        self.service.remove_tag(self.instance, self.tag1)

        mock_filter.assert_called_once_with(tag=self.tag1, content_object=self.instance)
        mock_queryset.delete.assert_called_once()

    @patch("flexi_tag.models.Tag.objects.filter")
    def test_get_tags(self, mock_filter):
        expected_result = MagicMock(spec=QuerySet)
        mock_filter.return_value = expected_result

        result = self.service.get_tags(self.instance)

        mock_filter.assert_called_once_with(tagged_items__content_object=self.instance)
        self.assertEqual(result, expected_result)

    def test_filter_by_tag(self):
        self.service.filter_by_tag(self.queryset, self.tag1.id)

        self.queryset.filter.assert_called_once_with(tagged_items__tag_id=self.tag1.id)

    def test_filter_by_tag_name(self):
        tag_name = "test-tag-1"
        self.service.filter_by_tag_name(self.queryset, tag_name)

        self.queryset.filter.assert_called_once_with(tagged_items__name=tag_name)



from django.test import TestCase
from mock import MagicMock, patch
from rest_framework import status
from rest_framework.parsers import JSONParser
from rest_framework.request import Request
from rest_framework.test import APIRequestFactory

from flexi_tag.exceptions import (
    TagIDRequiredException,
    TagIDsRequiredException,
    TagNotFoundException,
)
from flexi_tag.models import Tag
from flexi_tag.utils.views import TaggableViewSetMixin


class DummyTaggableViewSet(TaggableViewSetMixin):
    """
    A dummy ViewSet that inherits from TaggableViewSetMixin for testing purposes
    """

    def get_object(self):
        return MagicMock()

    def get_queryset(self):
        return MagicMock()

    def get_serializer(self, *args, **kwargs):
        serializer = MagicMock()
        serializer.data = {"id": 1, "name": "test"}
        return serializer

    def paginate_queryset(self, queryset):
        return None


class TaggableViewSetMixinTestCase(TestCase):
    def setUp(self):
        self.viewset = DummyTaggableViewSet()
        self.factory = APIRequestFactory()

        self.mock_taggable_service = MagicMock()
        self.viewset.taggable_service = self.mock_taggable_service

    def _convert_to_drf_request(self, request):
        """Helper method to convert Django request to DRF Request"""
        return Request(request, parsers=[JSONParser()])

    @patch("flexi_tag.utils.views.Tag.objects.get")
    def test_add_tag_success(self, mock_tag_get):
        mock_tag = MagicMock()
        mock_tag_get.return_value = mock_tag
        django_request = self.factory.post("/", {"tag_id": 1}, format="json")
        request = self._convert_to_drf_request(django_request)

        response = self.viewset.add_tag(request, pk=1)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.viewset.taggable_service.add_tag.assert_called_once()

    def test_add_tag_missing_tag_id(self):
        django_request = self.factory.post("/", {}, format="json")
        request = self._convert_to_drf_request(django_request)

        with self.assertRaises(TagIDRequiredException):
            self.viewset.add_tag(request, pk=1)

    @patch("flexi_tag.utils.views.Tag.objects.get", side_effect=Tag.DoesNotExist)
    def test_add_tag_not_found(self, mock_tag_get):
        django_request = self.factory.post("/", {"tag_id": 999}, format="json")
        request = self._convert_to_drf_request(django_request)

        with self.assertRaises(Tag.DoesNotExist):
            self.viewset.add_tag(request, pk=1)

    @patch("flexi_tag.utils.views.Tag.objects.get")
    def test_remove_tag_success(self, mock_tag_get):
        mock_tag = MagicMock()
        mock_tag_get.return_value = mock_tag
        django_request = self.factory.post("/", {"tag_id": 1}, format="json")
        request = self._convert_to_drf_request(django_request)

        response = self.viewset.remove_tag(request, pk=1)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.viewset.taggable_service.remove_tag.assert_called_once()

    def test_remove_tag_missing_tag_id(self):
        django_request = self.factory.post("/", {}, format="json")
        request = self._convert_to_drf_request(django_request)

        with self.assertRaises(TagIDRequiredException):
            self.viewset.remove_tag(request, pk=1)

    @patch("flexi_tag.utils.views.Tag.objects.get", side_effect=Tag.DoesNotExist)
    def test_remove_tag_not_found(self, mock_tag_get):
        django_request = self.factory.post("/", {"tag_id": 999}, format="json")
        request = self._convert_to_drf_request(django_request)

        with self.assertRaises(Tag.DoesNotExist):
            self.viewset.remove_tag(request, pk=1)

    def test_filter_by_tag_success(self):
        django_request = self.factory.get("/?tag_id=1")
        request = self._convert_to_drf_request(django_request)
        filtered_queryset = MagicMock()
        self.mock_taggable_service.filter_by_tag.return_value = filtered_queryset

        response = self.viewset.filter_by_tag(request)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.mock_taggable_service.filter_by_tag.assert_called_once()

    def test_filter_by_tag_missing_tag_id(self):
        django_request = self.factory.get("/")
        request = self._convert_to_drf_request(django_request)

        with self.assertRaises(TagIDRequiredException):
            self.viewset.filter_by_tag(request)

    def test_filter_by_tag_name_success(self):
        django_request = self.factory.get("/?tag_name=test")
        request = self._convert_to_drf_request(django_request)
        filtered_queryset = MagicMock()
        self.mock_taggable_service.filter_by_tag_name.return_value = filtered_queryset

        response = self.viewset.filter_by_tag_name(request)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.mock_taggable_service.filter_by_tag_name.assert_called_once()

    def test_filter_by_tag_name_missing_tag_name(self):
        django_request = self.factory.get("/")
        request = self._convert_to_drf_request(django_request)

        with self.assertRaises(TagIDsRequiredException):
            self.viewset.filter_by_tag_name(request)

    @patch("flexi_tag.utils.views.Tag.objects.get")
    def test_add_tag_none_tag(self, mock_tag_get):
        mock_tag_get.return_value = None
        django_request = self.factory.post("/", {"tag_id": 1}, format="json")
        request = self._convert_to_drf_request(django_request)

        with self.assertRaises(TagNotFoundException):
            self.viewset.add_tag(request, pk=1)

    @patch("flexi_tag.utils.views.Tag.objects.get")
    def test_remove_tag_none_tag(self, mock_tag_get):
        mock_tag_get.return_value = None
        django_request = self.factory.post("/", {"tag_id": 1}, format="json")
        request = self._convert_to_drf_request(django_request)

        with self.assertRaises(TagNotFoundException):
            self.viewset.remove_tag(request, pk=1)



from rest_framework import status
from rest_framework.decorators import action
from rest_framework.response import Response

from flexi_tag.exceptions import (
    TagIDRequiredException,
    TagIDsRequiredException,
    TagNotFoundException,
)
from flexi_tag.models import Tag
from flexi_tag.utils.service import TaggableService


class TaggableViewSetMixin(object):
    taggable_service = TaggableService()

    @action(detail=True, methods=["post"])
    def add_tag(self, request, pk=None):
        obj = self.get_object()
        tag_id = request.data.get("tag_id")

        if not tag_id:
            raise TagIDRequiredException()

        tag = Tag.objects.get(id=tag_id)
        if not tag:
            raise TagNotFoundException()

        self.taggable_service.add_tag(obj, tag)
        return Response(status=status.HTTP_200_OK)

    @action(detail=True, methods=["post"])
    def remove_tag(self, request, pk=None):
        obj = self.get_object()
        tag_id = request.data.get("tag_id")

        if not tag_id:
            raise TagIDRequiredException()

        tag = Tag.objects.get(id=tag_id)
        if not tag:
            raise TagNotFoundException()

        self.taggable_service.remove_tag(obj, tag)
        return Response(status=status.HTTP_200_OK)

    @action(detail=False, methods=["get"])
    def filter_by_tag(self, request):
        tag_id = request.query_params.get("tag_id")

        if not tag_id:
            raise TagIDRequiredException()

        queryset = self.taggable_service.filter_by_tag(
            queryset=self.get_queryset(),
            tag_id=tag_id,
        )

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @action(detail=False, methods=["get"])
    def filter_by_tag_name(self, request):
        tag_name = request.query_params.get("tag_name")

        if not tag_name:
            raise TagIDsRequiredException()

        queryset = self.taggable_service.filter_by_tag_name(
            queryset=self.get_queryset(),
            tag_name=tag_name,
        )

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)



from rest_framework import serializers

from flexi_tag.models import Tag, TaggedItem


class TagSerializer(serializers.ModelSerializer):
    class Meta:
        model = Tag
        fields = ("name",)
        read_only_fields = (
            "id",
            "modified_date",
            "created_date",
        )


class TaggedItemSerializer(serializers.ModelSerializer):
    tag_name = serializers.CharField(source="tag.name", read_only=True)

    class Meta:
        model = TaggedItem
        fields = ("id", "tag", "tag_name", "content_type", "object_id", "name")
        read_only_fields = ("id", "name", "tag_name")



from rest_framework import viewsets
from rest_framework.permissions import IsAdminUser
from rest_framework.viewsets import ReadOnlyModelViewSet

from flexi_tag.models import Tag, TaggedItem
from flexi_tag.resources.serializers import TaggedItemSerializer, TagSerializer
from flexi_tag.service import TagService


class TagViewSet(viewsets.ModelViewSet):
    queryset = Tag.objects.all()
    serializer_class = TagSerializer
    service = TagService()
    ordering_fields = "__all__"
    filterset_fields = ["name"]
    tag_service = TagService()
    permission_classes = (IsAdminUser,)

    def perform_create(self, serializer):
        serializer.instance = self.tag_service.create_instance(
            name=serializer.validated_data.get("name"),
        )

    def perform_update(self, serializer):
        serializer.instance = self.tag_service.update_instance(
            instance=serializer.instance,
            name=serializer.validated_data.get("name"),
        )

    def perform_destroy(self, instance):
        self.tag_service.delete_instance(instance)


class TaggedItemViewSet(ReadOnlyModelViewSet):
    queryset = TaggedItem.objects.all()
    serializer_class = TaggedItemSerializer
    permission_classes = (IsAdminUser,)
    ordering_fields = "__all__"
    filterset_fields = ["tag", "content_type", "object_id", "name"]



from django.contrib import admin

from flexi_tag.models import Tag, TaggedItem


@admin.register(Tag)
class TagAdmin(admin.ModelAdmin):
    list_display = (
        "id",
        "name",
        "created_date",
        "modified_date",
    )


@admin.register(TaggedItem)
class TaggedItemAdmin(admin.ModelAdmin):
    list_display = (
        "id",
        "tag",
        "content_type",
        "object_id",
        "name",
        "created_date",
        "modified_date",
    )



from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from flexi_tag.models import Tag, TaggedItem
from flexi_tag.resources.serializers import TaggedItemSerializer, TagSerializer


class TagSerializerTestCase(TestCase):
    def setUp(self):
        self.tag = Tag.objects.create(name="Existing Tag")
        self.serializer = TagSerializer(instance=self.tag)

    def test_contains_expected_fields(self):
        data = self.serializer.data
        self.assertEqual(set(data.keys()), {"name"})

    def test_name_field_content(self):
        data = self.serializer.data
        self.assertEqual(data["name"], "Existing Tag")

    def test_validate_empty_name(self):
        serializer = TagSerializer(data={"name": ""})
        self.assertFalse(serializer.is_valid())
        self.assertIn("name", serializer.errors)


class TaggedItemSerializerTestCase(TestCase):
    def setUp(self):
        self.tag = Tag.objects.create(name="Test Tag")
        self.content_type = ContentType.objects.create(
            app_label="test_app", model="test_model"
        )
        self.tagged_item = TaggedItem.objects.create(
            tag=self.tag,
            content_type=self.content_type,
            object_id=1,
            name=self.tag.name,
        )
        self.serializer = TaggedItemSerializer(instance=self.tagged_item)

    def test_contains_expected_fields(self):
        data = self.serializer.data
        expected_fields = {"id", "tag", "tag_name", "content_type", "object_id", "name"}
        self.assertEqual(set(data.keys()), expected_fields)

    def test_tag_name_field_content(self):
        data = self.serializer.data
        self.assertEqual(data["tag_name"], "Test Tag")

    def test_read_only_fields(self):
        data = {"id": "new_id", "name": "new_name", "tag_name": "new_tag_name"}
        serializer = TaggedItemSerializer(self.tagged_item, data=data, partial=True)
        self.assertTrue(serializer.is_valid())



import uuid

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="Tag",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("created_date", models.DateTimeField(auto_now_add=True)),
                ("modified_date", models.DateTimeField(auto_now=True, db_index=True)),
                ("name", models.CharField(db_index=True, max_length=255, unique=True)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="TaggedItem",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("created_date", models.DateTimeField(auto_now_add=True)),
                ("modified_date", models.DateTimeField(auto_now=True, db_index=True)),
                ("object_id", models.PositiveIntegerField()),
                ("name", models.CharField(db_index=True, max_length=255)),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                    ),
                ),
                (
                    "tag",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="tagged_items",
                        to="flexi_tag.tag",
                    ),
                ),
            ],
            options={
                "unique_together": {("tag", "content_type", "object_id")},
            },
        ),
    ]



from flexi_tag.exceptions import TagValidationException
from flexi_tag.models import Tag


class TagService:
    @staticmethod
    def create_instance(name):
        if Tag.objects.filter(name=name).exists():
            raise TagValidationException(name=name)

        return Tag.objects.create(
            name=name,
        )

    @staticmethod
    def update_instance(instance, name):
        if Tag.objects.filter(name=name).exclude(pk=instance.pk).exists():
            raise TagValidationException(name=name)

        if instance.tagged_items.exists():
            instance.tagged_items.update(name=name)

        instance.name = name
        instance.save(update_fields=["name"])
        return instance

    @staticmethod
    def delete_instance(instance):
        instance.delete()



import uuid

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models


class StarterModel(models.Model):
    id = models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True)
    created_date = models.DateTimeField(auto_now_add=True)
    modified_date = models.DateTimeField(auto_now=True, db_index=True)

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        update_fields = kwargs.pop("update_fields", None)
        if update_fields is not None:
            if not isinstance(update_fields, list):
                update_fields = list(update_fields)
            update_fields.append("modified_date")
            kwargs["update_fields"] = update_fields
        super(StarterModel, self).save(*args, **kwargs)


class Tag(StarterModel):
    name = models.CharField(max_length=255, unique=True, db_index=True)

    def __str__(self):
        return self.name


class TaggedItem(StarterModel):
    tag = models.ForeignKey(Tag, on_delete=models.CASCADE, related_name="tagged_items")
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey("content_type", "object_id")
    name = models.CharField(
        max_length=255, db_index=True
    )  # This field fills from Tag's name

    class Meta:
        unique_together = (
            "tag",
            "content_type",
            "object_id",
        )



from flexi_tag.models import Tag, TaggedItem


class TaggableService:
    @staticmethod
    def add_tag(instance, tag):
        TaggedItem.objects.create(tag=tag, content_object=instance, name=tag.name)

    @staticmethod
    def remove_tag(instance, tag):
        TaggedItem.objects.filter(tag=tag, content_object=instance).delete()

    @staticmethod
    def get_tags(instance):
        return Tag.objects.filter(
            tagged_items__content_object=instance,
        )

    @staticmethod
    def filter_by_tag(queryset, tag_id):
        return queryset.filter(tagged_items__tag_id=tag_id)

    @staticmethod
    def filter_by_tag_name(queryset, tag_name):
        return queryset.filter(tagged_items__name=tag_name)



from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase, override_settings
from rest_framework import status
from rest_framework.test import APIClient

from flexi_tag.models import Tag, TaggedItem


@override_settings(ROOT_URLCONF="flexi_tag.urls")
class TagViewSetTestCase(TestCase):
    def setUp(self):
        self.client = APIClient()
        self.admin_user = User.objects.create_superuser(
            username="admin", email="admin@example.com", password="admin123"
        )
        self.client.force_authenticate(user=self.admin_user)

        self.tag = Tag.objects.create(
            name="test_tag",
        )

        self.list_url = "/tags/"
        self.detail_url = "/tags/" + str(self.tag.id) + "/"

    def test_list_tags(self):
        response = self.client.get(self.list_url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 1)

    def test_retrieve_tag(self):
        response = self.client.get(self.detail_url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data["name"], "test_tag")

    def test_create_tag(self):
        data = {
            "name": "new_tag",
        }
        response = self.client.post(self.list_url, data, format="json")
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(Tag.objects.count(), 2)
        self.assertEqual(response.data["name"], "new_tag")

    def test_update_tag(self):
        data = {
            "name": "updated_tag",
        }
        response = self.client.put(self.detail_url, data, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data["name"], "updated_tag")

    def test_delete_tag(self):
        response = self.client.delete(self.detail_url)
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
        self.assertEqual(Tag.objects.count(), 0)

    def test_unauthorized_access(self):
        self.client.force_authenticate(
            user=User.objects.create_user(
                username="regular_user", password="regular123"
            )
        )

        response = self.client.get(self.list_url)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


@override_settings(ROOT_URLCONF="flexi_tag.urls")
class TaggedItemViewSetTestCase(TestCase):
    def setUp(self):
        self.client = APIClient()
        self.admin_user = User.objects.create_superuser(
            username="admin2", email="admin2@example.com", password="admin123"
        )
        self.client.force_authenticate(user=self.admin_user)

        self.tag = Tag.objects.create(name="test_tag")
        self.content_type = ContentType.objects.create(
            app_label="test_app", model="test_model"
        )
        self.tagged_item = TaggedItem.objects.create(
            tag=self.tag,
            content_type=self.content_type,
            object_id=1,
            name=self.tag.name,
        )

        self.list_url = "/tagged-items/"
        self.detail_url = "/tagged-items/" + str(self.tagged_item.id) + "/"

    def test_list_tagged_items(self):
        response = self.client.get(self.list_url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 1)

    def test_retrieve_tagged_item(self):
        response = self.client.get(self.detail_url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data["name"], self.tag.name)
        self.assertEqual(response.data["tag_name"], self.tag.name)

    def test_create_tagged_item_not_allowed(self):
        data = {
            "tag": self.tag.id,
            "content_type": self.content_type.id,
            "object_id": 2,
            "name": "test",
        }
        response = self.client.post(self.list_url, data, format="json")
        self.assertEqual(response.status_code, status.HTTP_405_METHOD_NOT_ALLOWED)

    def test_unauthorized_access(self):
        self.client.force_authenticate(
            user=User.objects.create_user(
                username="regular_user", password="regular123"
            )
        )
        response = self.client.get(self.list_url)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
